;;
;; ENTITY AI CONTROL SYSTEM
;; Controla el sistema de IA
;;
.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "cpctelera.h.s"
.include "man/patrol.h.s"

.module sys_ai_control


;; Inicializacion sistema de ia
;; Obtiene el puntero al vector de entidades para guardarlo
sys_ai_control_init::
	ld (_ent_array_ptr), hl
	ret


sys_ai_stand_by::

	ld e_ai_last_st(ix), #e_ai_st_stand_by
	ld e_ai_st(ix), #e_ai_st_patrol
	ret

;; Metodo para moverse a una posicion
;; De momento se mueve a donde esta el personaje al pulsar espacio
sys_ai_move_to:

;; Modifico posicion X
	ld a, e_ai_aim_x(ix) ;; A = obj_x
	sub e_x(ix) ;; A = obj_x - x
	jr nc, objx_greater_or_equal ;; if(obj_x - x > 0) --> objx_greater_or_equal

_objx_lesser:
	ld e_vx(ix), #-1
	jr _endif_x
	

objx_greater_or_equal:
	jr z, _arrived_x ;; Si es 0 he llegado al objetivo
	ld e_vx(ix), #1
	jr _endif_x

_arrived_x:
	ld e_vx(ix), #0 ;; Modifico la velocidad X a 0

_endif_x:
	

;; Modifico posicion Y
	ld a, e_ai_aim_y(ix) ;; A = obj_y
	sub e_y(ix) ;; A = obj_y - y
	jr nc, objy_greater_or_equal ;; if(obj_y - y > 0) --> objy_greater_or_equal

_objy_lesser:
	ld e_vy(ix), #-1
	jr _endif_y

objy_greater_or_equal:
	jr z, _arrived_y ;; Si es 0 he llegado al objetivo
	ld e_vy(ix), #1
	jr _endif_y

_arrived_y:
	ld e_vy(ix), #0 ;; Modifico la velocidad Y a 0

	;; Comprobar antes si la X tambien es 0 para modificar el estado a e_ai_st_stand_by
	ld a, e_vx(ix) ;; Cargo en A la velocidad de X
	or a ;; Si A es 0 se activa el flag Zero
	jr nz, _endif_y ;; Salto si no se activa el flag Zero

	;; Si la velocidad de A es 0 he llegado en X y Y y cambio e l estado
	ld a, e_ai_last_st(ix)
	ld e_ai_st(ix), a
	ld e_ai_last_st(ix), #e_ai_st_move_to
	;ld e_ai_st(ix), #e_ai_st_stand_by
	;ld e_ai_last_st(ix), #sys_ai_move_to

_endif_y:
	ret


sys_ai_control_update::
	;ld (_ent_counter), a ;; Almaceno el numero de entidades

_ent_array_ptr = . + 2 ;; Constante que apunta al array de entidades
	;; Sobreescribo el valor #0x0000 con el puntero al array de entidades
	ld ix, #0x0000 ;; La instruccion ocupa 4 bytes, 2 para ld ix y 2 para #0x0000

_loop:

	
	ld a, e_w(ix) ;; Cargo en A el estado de ia de la entidad
	cp #e_w_invalidEntity
	ret z
	;; Compruebo si la entidad tiene IA o no
	ld a, e_ai_st(ix)
	;; #e_ai_st_noAI --> Al ser un valor inmediato se accede con #
	cp #e_ai_st_noAI
	jr z, _no_AI_ent  ;; Comparo los valores A con e_ai_st_noAI --> Si son iguales quiere decir que no tiene IA y no debe actualizarla

	cp #e_ai_door
	jr z, _no_AI_ent

	cp #e_ai_last_door
	jr z, _no_AI_ent

;; Entidades con IA
_AI_ent:
	;; Compruebo si es e_ai_st_stand_by
	cp #e_ai_st_stand_by
	call z, sys_ai_stand_by

	;; Compruebo si es e_ai_st_move_to
	cp #e_ai_st_move_to
	call z, sys_ai_move_to

	cp #e_ai_st_patrol
	call z, sys_ai_patrol

;; Entidades sin IA
_no_AI_ent:
	ld de, #sizeof_e
	add ix, de;Siguinte entidad

	jr _loop



;Patrol
sys_ai_patrol::
;	
;Direccion de memoria dond estan las coordendas de la patrulla	
	ld l, e_ai_patrol_step_l(ix)
	ld h, e_ai_patrol_step_h(ix)

	;x objetivo patrol -- si x == -1 hemos termnado las coordenadas
	ld a, (hl) ;
	cp #patrol_invalid_move_x
	jr z, _reset_patrol


	ld e_ai_aim_x(ix),a
	inc hl
	ld a, (hl) ;
	ld e_ai_aim_y(ix),a
	
	inc hl
	ld e_ai_patrol_step_l(ix), l
	ld e_ai_patrol_step_h(ix), h
	;Cambio de estado
 	ld e_ai_last_st(ix), #e_ai_st_patrol
	ld e_ai_st(ix), #e_ai_st_move_to
;
	ret

_reset_patrol:

	inc hl
	ld  a, (hl)
	inc hl
	ld  h,(hl)

	ld  e_ai_patrol_step_l(ix), a
	ld  e_ai_patrol_step_h(ix), h


	ret