;;
;; ENTITY COLLISIONS SYSTEM
;; Detecta colisiones entre el jugador y los npcs
;;
;; IMPORTANTE

.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "man/game.h.s"
.include "cpctelera.h.s"

.module sys_entity_collisions


_hero_coordinates:
	.db #1				;; Coordenada x del heroe				
	.db #1				;; Coordenada y del heroe
	.db #1 				;; Suma de heroe_x + heroe_w
	.db #1				;; Suma de heroe_y + heroe_h
	.db #1				;; Contador de entidades que quedan por tratar


sys_collisions_init::
	ret


sys_collisions_update::

	dec a 				;; Quito uno al numero de entidades porque no cuento al jugador para hacer el bucle
	ld b, a 				;; Cargo numero de entidades en b

	ld hl, #_hero_coordinates	;; Cargo en hl la direccion de memoria donde almacenaremos las coordenadas del heroe

	ld a, e_x(ix)   			;; Cargo la x del heroe en a
	ld (hl), a				;; Cargo la x del heroe en la estructura
	
	inc hl

	ld a, e_y(ix)			;; Cargo la y del heroe en a
	ld (hl), a				;; Cargo la y del heroe en la estructura

	inc hl
	
	ld a, e_x(ix) 			;; Cargo la x de heroe en a
	add e_w(ix)				;; Le añado su anchura
	ld (hl), a 				;; Cargo la suma de heroe_x + heroe_w en la estructura

	inc hl

	ld a, e_y(ix) 			;; Cargo la y de heroe en a
	add e_h(ix)				;; Le añado su altura
	ld (hl), a 				;; Cargo la suma de heroe_y + heroe_h en la estructura

	dec hl
	dec hl				;; Decremento hl y lo dejo apuntando a la y del heroe

	ld de, #sizeof_e 			;; Almaceno en de el tamaño de la entidad 
	add ix, de 				;; Sumo de a ix para ir al primer NPC

	
	
_collisions_loop:

;; COMPROBAR HEROE_Y < SUMA_ENTIDAD_Y

_first_check_y:
	ld a, e_y(ix) 			;; Cargo en a la coordenada y del NPC
	add e_h(ix) 			;; Le sumo su altura

	ld d, a 				;; Cargo en d la suma de la posicion y del npc con su altura
	ld a, (hl)				;; Cargo en a la coordenada y del heroe 
	inc d
	cp d					;; Compruebo si la y del heroe es menor que la suma_entidad_y 
	jr nc, _next_entity 		;; Si a es mayor que d (a > d) salto a _next_entity 

;; COMPROBAR SUMA_HEROE_Y > ENTIDAD_Y

_second_check_y:
	inc hl
	inc hl				;; Dejo hl apuntando hacia la suma_heroe_y

	ld a, (hl)				;; Pongo en a la suma_heroe_y
	ld d, e_y(ix)			;; Pongo en d la coordenada y del NPC
	inc a
	cp d					;; Compruebo si la suma_heroe_y es mayor que la coordenada y de la entidad
	jr c, _next_entity		;; Si suma_heroe_y es menor que entidad_y salto a _next_entity

;; COMPROBAR HEROE_X < SUMA_ENTIDAD_X

_first_check_x:
	
	dec hl				
	dec hl
	dec hl				;; Dejo hl apuntando hacia la coordenada x del heroe

	ld a, e_x(ix) 			;; Cargo en a la x del NPC
	add e_w(ix) 			;; Le sumo su anchura
	ld d, a				;; Cargo en d la suma de la posicion x del npc con su anchura
	inc d
	ld a, (hl)				;; Cargo en a la x del heroe 
	cp d					;; Compruebo si la suma es menor que la posicion x del héroe 
	jr nc, _next_entity 		;; Si a es mayor que d (a > d) salto a _next_entity

;; COMPROBAR SUMA_HEROE_X > ENTIDAD_X

_second_check_x:
	inc hl
	inc hl				;; Dejo hl apuntando hacia la suma_heroe_x

	ld a, (hl)				;; Pongo en a la suma_heroe_x
	ld d, e_x(ix)			;; Pongo en d la coordenada x del NPC
	inc a
	cp d					;; Compruebo si la suma_heroe_x es mayor que la coordenada x de la entidad
	dec hl				;; Dejo hl apuntando a la coordenada y del heroe
	jr nc, _collision			;; Si suma_heroe_x es mayor que entidad_y salto a _collision


_next_entity:

	dec b
	ret z

	ld de, #sizeof_e 			
	add ix, de

	jr _collisions_loop


_collision:
	ld a, e_ai_st(ix)
	cp #e_ai_door
	jr z, _collision_door

	cp #e_ai_last_door
	jr z, _collision_last_door
	
	ld a, #_end_game
	ret

_collision_door:
	ld a, #_end_game_door
	ret

_collision_last_door:
	ld a, #_end_game_last_door
	ret