;;
;; SQUARE RENDER SYSTEM
;; Renders Square components
;;

.include "cmp/entity.h.s"
.include "man/entity.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "assets/assets.h.s"


.module sys_entity_render



screen_start = 0xC000 ;; Indico donde empieza la memoria de video
string: .asciz "PRESS SPACE TO PLAY!";

sys_eren_init::
	ld (_ent_array_ptr), hl
	

	ld c, #0 ;; Pongo el modo de video a 0 --> (Mode 0)
	call cpct_setVideoMode_asm ;; Cojo la paleta de colores
	ld hl, #_pal_main ;; Cargo en hl la paleta principal
	ld de, #16 ;; Cargo en DE 16 que es el tamaño de la paleta
	call cpct_setPalette_asm ;; Llamo a setPalette
	cpctm_setBorder_asm HW_WHITE ;; Pongo el borde de color gris
	ret


starting_screen::
	
	ld   hl, #_initGame_z_end
	ld   de, #0xFFFF
	call cpct_zx7b_decrunch_s_asm
	
	;; Set up draw char colours before calling draw string
	;ld    d, #0         ;; D = Background PEN (0)
	;ld    e, #3         ;; E = Foreground PEN (3)
;
;	;call cpct_setDrawCharM1_asm   ;; Set draw char colours
;
;	;;; Calculate a video-memory location for printing a string
;	;ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
;	;ld    b, #150                 ;; B = y coordinate (24 = 0x18)
;	;ld    c, #16                  ;; C = x coordinate (16 = 0x10)
;
;	;call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
;
;	;;; Print the string in video memory
;	;;; HL already points to video memory, as it is the return
;	;;; value from cpct_getScreenPtr_asm
;	;ld   iy, #string    ;; IY = Pointer to the string 
;
	;call cpct_drawStringM1_asm  ;; Draw the string

_loop_PressAnyKey:

	call cpct_scanKeyboard_f_asm
	ld hl, #Key_Space
	call cpct_isKeyPressed_asm
	;call cpct_isAnyKeyPressed_asm

	jr z, _loop_PressAnyKey 

	ret

dead_screen::
	
	ld   hl, #_gameOver_z_end
	ld   de, #0xFFFF
	call cpct_zx7b_decrunch_s_asm
	jr _loop_PressAnyKey 

victorius_screen::
	ld   hl, #_finishGame_z_end
	ld   de, #0xFFFF
	call cpct_zx7b_decrunch_s_asm
	jr _loop_PressAnyKey 


print_level::
	;ARRAY DE TILES
	ld   hl, #_array_tiles ;Puntero al array de tiles (PATRONES PARA CONSTRUR EL MAPA)
	call cpct_etm_setTileset2x4_asm
	
	;CARGA DEL MAPA
	;Always received on the stack
	;(2B) pvideomem	Pointer to upper left corner of the tilemap in video memory.
	;(2B) ptilemap	Pointer to the start of the tilemap


_selection_start:

	ld hl, #_level_counter
	ld c, (hl)			;;Almaceno el contador de nivel

	ld hl, #_levels		;; Almaceno en hl el puntero al array de niveles

_selection_loop:

_check_invalid_level:	
	ld a, (hl)
	cp #invalid_level
	jr nz, _level_ok

	call starting_screen

	ld hl, #_level_counter
	ld (hl), #1

	ld hl, #_levels
	ld c, #1

_level_ok:
	ld e, (hl)
	inc hl
	ld d, (hl)			;; Cargo en DE lo que hay en el array de niveles
	

	dec c				;; Decremento el contador
	inc hl			;; Incremento el puntero al array de niveles

	ld b, #0
	ld a, c
	cp b				;; Si no he terminado el bucle vuelvo
	jr nz, _selection_loop 



_level_selection:

	ex de, hl			;; Cargo en hl la direccion de memoria al nivel que se tiene que cargar
	

_level_selected:


	;decompress_buffer_tilemap     = 0x350
	;sizeTilemap                   = 0x7D0 ;(2000 Bytes) 7D0h
	;decompress_buffer_tilemap_end == decompress_buffer_tilemap + sizeTilemap -1
	
	;en hl recibimos la info necesria
	ld   de, #decompress_buffer_tilemap_end
	call cpct_zx7b_decrunch_s_asm;Funciona

	ld hl, #0x350
	push hl

	ld   hl, #0xC000
	push hl
	


	;(1B C) x	x tile-coordinate of the starting tile inside the tilemap
	;(1B B) y	y tile-coordinate of the starting tile inside the tilemap
	;(1B E) w	width in tiles of the tile-box to be redrawn
	;(1B D) h	height in tiles of the tile-box to be redrawn
	;(1B A) map_width	Width in tiles of a complete row of the tilemap	
	ld   bc, #0000
	ld   de, #0x3228
	ld    a, #40
	call cpct_etm_drawTileBox2x4_asm
	
	ret

;print_level_door::
;	;ARRAY DE TILES
;	ld   hl, #_array_tiles ;Puntero al array de tiles (PATRONES PARA CONSTRUR EL MAPA)
;	call cpct_etm_setTileset2x4_asm
;	
;	;CARGA DEL MAPA
;	;Always received on the stack
;	;(2B) pvideomem	Pointer to upper left corner of the tilemap in video memory.
;	;(2B) ptilemap	Pointer to the start of the tilemap
;	ld   hl, #_level3
;	push hl
;	ld   hl, #0xC000
;	push hl
;
;	;(1B C) x	x tile-coordinate of the starting tile inside the tilemap
;	;(1B B) y	y tile-coordinate of the starting tile inside the tilemap
;	;(1B E) w	width in tiles of the tile-box to be redrawn
;	;(1B D) h	height in tiles of the tile-box to be redrawn
;	;(1B A) map_width	Width in tiles of a complete row of the tilemap	
;	ld   bc, #0000
;	ld   de, #0x3228
;	ld    a, #40
;	call cpct_etm_drawTileBox2x4_asm
;	
;	ret


asignedFirstlastVP::
	call man_entity_getArray
loopFirstlastVP:

	ld a, e_w(ix) ;; Cargo en A el estado de ia de la entidad
	cp #e_w_invalidEntity
	ret z
	
	ld de, #screen_start
	ld c, e_x(ix)
	ld b, e_y(ix)
	call cpct_getScreenPtr_asm ;; Calculo la posicion de memoria donde pintar la entidad

	ld e_lastVP_l(ix), l
	ld e_lastVP_h(ix), h

	ld de, #sizeof_e
	add ix, de;Siguinte entidad
	jr loopFirstlastVP


;eraseSprites:
;	call man_entity_getArray
;_erase_loop:
;	ld a, e_w(ix) ;; Cargo en A el estado de ia de la entidad
;	cp #e_w_invalidEntity
;	ret z
;
;	;(2B DE) memory	Video memory pointer to the upper left box corner byte
;	;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
;	;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
;	;(1B B ) height	Box height in bytes (>0)
;	ld   a, #0xFF
;	ld   e, e_lastVP_l(ix) ;; Cargo en e la ultima posicion de memoria
;	ld   d, e_lastVP_h(ix) ;; Cargo en d la ultima posicion de memoria  
;	ld   c, e_w(ix) ;; Cargo en c el ancho
;	ld   b, e_h(ix) ;; Cargo en b el alto
;	;push bc ;; Almaceno en la pila bc
;	call cpct_drawSolidBox_asm
;	ld de, #sizeof_e
;	add ix, de;Siguinte entidad
;	jr _erase_loop

sys_eren_update::
	;call eraseSprites
	call sys_eren_render_entities ;; Llamo a renderizar las entidades
	;; En un futuro tendre de pitnar el mapa de fondo y mas cosas
	ret

;; INPUT
;;	IX = Pinter to the first entity to render

sys_eren_render_entities::
	;ld (_ent_counter), a ;; Almaceno en _ent_conter la cantidad de entidades
	_ent_array_ptr = . + 2 ;; Constante que apunta al array de entidades
	;; Sobreescribo el valor #0x0000 con el puntero al array de entidades
	ld ix, #0x0000 ;; La instruccion ocupa 4 bytes, 2 para ld ix y 2 para #0x0000

	ld de, #screen_start
	ld c, e_x(ix)
	ld b, e_y(ix)
	call cpct_getScreenPtr_asm ;; Calculo la posicion de memoria donde pintar la entidad
_update_loop:

	ld a, e_w(ix) ;; Cargo en A el estado de ia de la entidad
	cp #e_w_invalidEntity
	ret z
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;;	;;; Borro las entidades con el fondo anterior de la entidad
	;ld   l, e_ptr_bg_l(ix)
	;ld   h, e_ptr_bg_h(ix)
	;ld   e, e_lastVP_l(ix) ;; Cargo en e la ultima posicion de memoria
	;ld   d, e_lastVP_h(ix) ;; Cargo en d la ultima posicion de memoria  
	;ld   c, e_w(ix) ;; Cargo en c el ancho
	;ld   b, e_h(ix) ;; Cargo en b el alto
	;;push bc ;; Almaceno en la pila bc
	;call cpct_drawSprite_asm
	ld   a, #0x03
	ld   e, e_lastVP_l(ix) ;; Cargo en e la ultima posicion de memoria
	ld   d, e_lastVP_h(ix) ;; Cargo en d la ultima posicion de memoria  
	ld   c, e_w(ix) ;; Cargo en c el ancho
	ld   b, e_h(ix) ;; Cargo en b el alto
	push bc ;; Almaceno en la pila bc
	call cpct_drawSolidBox_asm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	


	;; Calcula la nueva posicion de memoria de video donde pintar la entidad
	ld de, #screen_start
	ld c, e_x(ix)
	ld b, e_y(ix)
	call cpct_getScreenPtr_asm ;; Calculo la posicion de memoria donde pintar la entidad
	;;; Almaceno la posicion calculada para mas tarde borrarla
	ld e_lastVP_l(ix), l
	ld e_lastVP_h(ix), h
	;hl recibido de cpct_getScreenPtr_asm; Posicion memoria video 
	;ld  e, e_ptr_bg_l(ix) ;Direccio donde almaceno el sprite
	;ld  d, e_ptr_bg_h(ix)	
	;ld   c, e_w(ix) ;; Cargo en c el ancho
	;ld   b, e_h(ix) ;; Cargo en b el alto
	;call cpct_getScreenToSprite_asm;Me guardo el fondo antes de pintar el sprite hero y no perder el fondo



	;; Cargo el sprite en funcion de su movimiento
	ld a, e_last_movement(ix)
	cp #01
	jr z, _sprite_derecha
	jr nz, _sprite_izquierda

_sprite_derecha:
	ld l, e_pspr_l_right(ix) ;; Almaceno en L la posicion de memoria donde el sprite tiene su definicion
	ld h, e_pspr_h_right(ix) ;; Almaceno en H la posicion de memoria donde el sprite tiene su definicion
	jr _continue

_sprite_izquierda:
	ld l, e_pspr_l_left(ix) ;; Almaceno en L la posicion de memoria donde el sprite tiene su definicion
	ld h, e_pspr_h_left(ix) ;; Almaceno en H la posicion de memoria donde el sprite tiene su definicion
	jr _continue


_continue:
	ld e, e_lastVP_l(ix)
	ld d, e_lastVP_h(ix)
	;ld   c, e_w(ix) ;; Cargo en c el ancho
	;ld   b, e_h(ix) ;; Cargo en b el alto
	pop bc
	call cpct_drawSprite_asm
	
	ld de, #sizeof_e
	add ix, de;Siguinte entidad

	jr _update_loop


;; ##### EXPLICACION DE LAS BUENAS #####
;; El . es donde el ensamblador va a generar el codigo, la posicion actual de memoria donde va a genera el siguiente byte
;; _ent_counter es una constante y tiene el valor de la posicion de memoria donde se va a generar el siguiente byte +1,
;; debido a que al entrar a la funcion almaceno en _ent_counter la cantidad de entidades
;; el siguiente byte es 3E y el siguiente +1 es el byte 00
;; Si la constante son 3 entidades sobreescribo el 00 con mi numero de entidades para posteriormente cargarlo en a
;_ent_counter = .+1 ;;
;	;; En maquina 3E 00
;	ld a, #0 ;; Cargo en A la cantidad de entidades
;	dec a ;; Decremento a --> a -= 1
;	ret z ;; SI es 0 no quedan entidades y retorno
;
;	;; Si quedan entidades por crear
;	ld (_ent_counter), a ;; Actualizo el contador _ent_counter
;	ld bc, #sizeof_e ;; Almaceno en bc el tamaño de la entidad --> bc = #sizeof_e
;	add ix, bc ;; Sumo bc a ix para ir a la siguiente entidad --> ix += bc
;	jr _update_loop




;; Se puede modificar el ent_counter poniendolo como etiqueta, que es la direccion de memoria
;; y donde se utilize _ent_counter hacer _ent_counter +1
